@@ -9,7 +9,7 @@
 -include("megaco_text_mini_parser.hrl").
 
 
--file("/build/erlang-nox/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
+-file("/build/erlang-nox-20/src/otp/bootstrap/lib/parsetools/include/yeccpre.hrl", 0).
 %%
 %% %CopyrightBegin%
 %%
